<!DOCTYPE html>
{% import 'summary_macros.html' as macros %}
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ report.proj_name }} - BLog Summary</title>

    <style>
        body {
          font-family: Arial, sans-serif;
          color: #0F172A;
          line-height: 1.6;
          background-color: #F8FAFC;
          padding: 20px;
        }

        table {
          width: 97%;
          border-collapse: collapse;
          margin-bottom: 20px;
          font-size: 14px;
        }

        th, td {
          border: 1px solid #E2E8F0;
          padding: 10px;
          text-align: left;
        }

        th {
          background-color: #F1F5F9;
          font-weight: bold;
        }

        .badge {
          display: inline-block;
          padding: 0px 8px;
          font-size: 14px;
          font-weight: bold;
          text-transform: uppercase;
          border-radius: 4px;
        }

        .passed {
          background-color: #D1FAE5;
        }

        .failed {
          background-color: #FECACA;
        }

        .broken {
          background-color: #FEF3C7;
        }

        .skipped {
          background-color: #F3F4F6;
        }

        .row.passed {
          background-color: #ECFDF5;
        }

        .row.failed {
          background-color: #FEF2F2;
        }

        .row.broken {
          background-color: #FFFBEB;
        }

        .row.skipped {
          background-color: #F9FAFB;
        }

        h3 {
          margin-top: 30px;
          margin-bottom: 10px;
          color: #1E293B;
        }

        .table-wrapper {
          width: 49%;
          display: inline-block;
          vertical-align: top;
        }

        @media (max-width: 768px) {
        .table-wrapper {
          width: 100%;
          display: block;
          margin-bottom: 1rem;
        }
    </style>
</head>

<body>
    <div class="container">
        {{ macros.render_full_report_summary(report) }}
    </div>
</body>
</html>